from typing import List

from rich import box
from rich.columns import Columns
from rich.console import Console
from rich.padding import Padding
from rich.table import Table
from rich.theme import Theme

info_theme = Theme(
    {
        "data": "#999966",
        "proj": "#8A2BE2",
        "comp": "#32CD32",
        "base": "#6495ED",
    }
)


def brick_status(brick, bricks) -> str:
    status = ":heavy_check_mark:" if brick in bricks else "-"

    return f"[data]{status}[/]"


def print_bricks_in_projects(
    projects_data: List[dict], bases: List[str], components: List[str]
) -> None:
    if not components and not bases:
        return

    console = Console(theme=info_theme)
    table = Table(box=box.SIMPLE_HEAD)
    table.add_column("[data]brick[/]")

    proj_cols = [f"[proj]{project['name']}[/]" for project in projects_data]
    table.add_column(Columns(proj_cols, align="center", expand=True))

    for brick in sorted(components):
        cols = [brick_status(brick, p.get("components")) for p in projects_data]
        table.add_row(f"[comp]{brick}[/]", Columns(cols, align="center", expand=True))

    for brick in sorted(bases):
        cols = [brick_status(brick, p.get("bases")) for p in projects_data]
        table.add_row(f"[base]{brick}[/]", Columns(cols, align="center", expand=True))

    console.print(table, overflow="ellipsis")


def print_workspace_summary(
    projects_data: List[dict], bases: List[str], components: List[str]
) -> None:
    console = Console(theme=info_theme)

    console.print(Padding("[data]Workspace summary[/]", (1, 0, 1, 0)))

    number_of_projects = len(projects_data)
    number_of_components = len(components)
    number_of_bases = len(bases)

    console.print(f"[proj]projects[/]: [data]{number_of_projects}[/]")
    console.print(f"[comp]components[/]: [data]{number_of_components}[/]")
    console.print(f"[base]bases[/]: [data]{number_of_bases}[/]")



from polylith.project.create import create_project
from polylith.project.get import (
    get_packages_for_projects,
    get_project_names,
    get_project_names_and_paths,
)
from polylith.project.parser import parse_package_paths

__all__ = [
    "create_project",
    "get_project_names",
    "get_project_names_and_paths",
    "get_packages_for_projects",
    "parse_package_paths",
]



from polylith.check.core import run_command
from rich.console import Console
from rich.theme import Theme

info_theme = Theme(
    {
        "data": "#999966",
        "proj": "#8A2BE2",
        "comp": "#32CD32",
        "base": "#6495ED",
    }
)


def run(project_data: dict) -> bool:
    console = Console(theme=info_theme)

    project_name = project_data["name"]
    project_path = project_data["path"]

    with console.status(f"checking [proj]{project_name}[/]", spinner="monkey"):
        result = run_command(project_path)

        message = ["[proj]", project_name, "[/]", " "]
        extra = [":warning:"] if result else [":heavy_check_mark:"]

        output = "".join(message + extra)
        console.print(output)

        for row in result:
            console.print(f"[data]{row}[/]")

        return True if not result else False



from polylith.info.collect import get_bases, get_bricks_in_projects, get_components
from polylith.info.report import print_bricks_in_projects, print_workspace_summary

__all__ = [
    "get_bases",
    "get_bricks_in_projects",
    "get_components",
    "print_bricks_in_projects",
    "print_workspace_summary",
]



from polylith.repo.repo import (
    bases_dir,
    components_dir,
    default_toml,
    development_dir,
    find_workspace_root,
    projects_dir,
    readme_file,
    workspace_file,
)

__all__ = [
    "bases_dir",
    "components_dir",
    "default_toml",
    "development_dir",
    "find_workspace_root",
    "projects_dir",
    "readme_file",
    "workspace_file",
]



import os
import subprocess
from pathlib import Path
from typing import List


def navigate_to(path: Path):
    os.chdir(str(path))


def run_command(project_path: Path) -> List[str]:
    current_dir = Path.cwd()

    navigate_to(project_path)

    try:
        res = subprocess.run(
            ["poetry", "check-project"], capture_output=True, text=True
        )
    finally:
        navigate_to(current_dir)

    res.check_returncode()

    return res.stdout.splitlines()



from pathlib import Path

from poetry.console.commands.command import Command
from polylith import check, project, repo


class CheckCommand(Command):
    name = "poly check"
    description = "Validates the <comment>Polylith</> workspace."

    def handle(self) -> int:
        root = repo.find_workspace_root(Path.cwd())
        if not root:
            raise ValueError(
                "Didn't find the workspace root. Expected to find a workspace.toml file."
            )

        projects = project.get_project_names_and_paths(root)

        res = [check.report.run(proj) for proj in projects]

        return 0 if all(res) else 1



from pathlib import Path

from poetry.console.commands.command import Command
from polylith import info, repo, workspace


class InfoCommand(Command):
    name = "poly info"
    description = "Info about the <comment>Polylith</> workspace."

    def handle(self) -> int:
        root = repo.find_workspace_root(Path.cwd())
        if not root:
            raise ValueError(
                "Didn't find the workspace root. Expected to find a workspace.toml file."
            )

        ns = workspace.parser.get_namespace_from_config(root)
        bases = info.get_bases(root, ns)
        components = info.get_components(root, ns)
        projects_data = info.get_bricks_in_projects(root, components, bases, ns)

        info.print_workspace_summary(projects_data, bases, components)
        info.print_bricks_in_projects(projects_data, bases, components)

        return 0



import subprocess
from pathlib import Path
from typing import List, Union

from polylith import repo, workspace


def _parse_brick_name(folder: str, changed_file: Path) -> str:
    file_path = Path(changed_file.as_posix().replace(folder, ""))

    return next(p for p in file_path.parts if p != file_path.root)


def _get_changed(folder: str, changed_files: List[Path]) -> set:
    return {_parse_brick_name(folder, f) for f in changed_files if folder in f.as_posix()}


def _get_changed_bricks(
    root: Path, top_dir: str, changed_files: List[Path], namespace: str
) -> list:
    d = f"{top_dir}/{namespace}"

    return sorted(_get_changed(d, changed_files))


def get_changed_components(
    root: Path, changed_files: List[Path], namespace: str
) -> list:
    return _get_changed_bricks(root, repo.components_dir, changed_files, namespace)


def get_changed_bases(root: Path, changed_files: List[Path], namespace: str) -> list:
    return _get_changed_bricks(root, repo.bases_dir, changed_files, namespace)


def get_changed_projects(changed_files: List[Path]) -> list:
    res = _get_changed(repo.projects_dir, changed_files)
    filtered = {p for p in res if p != repo.projects_dir}
    return sorted(filtered)


def get_latest_tag(root: Path) -> Union[str, None]:
    tag_pattern = workspace.parser.get_git_tag_pattern_from_config(root)

    res = subprocess.run(
        ["git", "tag", "-l", "--sort=-committerdate", f"{tag_pattern}"],
        capture_output=True,
    )

    return next((tag for tag in res.stdout.decode("utf-8").split()), None)


def get_files(tag: str) -> List[Path]:
    res = subprocess.run(
        ["git", "diff", tag, "--stat", "--name-only"],
        capture_output=True,
    )

    return [Path(p) for p in res.stdout.decode("utf-8").split()]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['polylith',
 'polylith.bricks',
 'polylith.check',
 'polylith.development',
 'polylith.diff',
 'polylith.dirs',
 'polylith.files',
 'polylith.info',
 'polylith.interface',
 'polylith.log',
 'polylith.poetry.commands',
 'polylith.poetry_plugin',
 'polylith.project',
 'polylith.readme',
 'polylith.repo',
 'polylith.test',
 'polylith.workspace']

package_data = \
{'': ['*']}

install_requires = \
['poetry>=1.2,<2.0', 'rich>=12.6.0,<13.0.0', 'tomlkit>=0.11.5,<0.12.0']

entry_points = \
{'poetry.application.plugin': ['poetry-polylith-plugin = '
                               'polylith.poetry_plugin:PolylithPlugin']}

setup_kwargs = {
    'name': 'poetry-polylith-plugin',
    'version': '1.1.0',
    'description': 'A Poetry plugin that adds tooling support for the Polylith Architecture',
    'long_description': '# Poetry Polylith Plugin\n\nThis is a Python `Poetry` plugin, adding CLI support for the Polylith Architecture.\n\n## Usage\n\n### Install Poetry & plugins\nWith the `Poetry` version 1.2 or later installed, you can add plugins.\n\nAdd the [Multiproject](https://github.com/DavidVujic/poetry-multiproject-plugin) plugin, that will enable the very important __workspace__ support (i.e. relative package includes) to Poetry.\n``` shell\npoetry self add poetry-multiproject-plugin\n```\n\nAdd the Polylith plugin:\n``` shell\npoetry self add poetry-polylith-plugin\n```\n\n### Create a repository\nCreate a directory for your code, initialize it with __git__ and create a basic __Poetry__ setup:\n\n``` shell\ngit init\n\npoetry init\n```\n\n### Commands\nThe `create workspace` command will create a Polylith workspace, with a basic Polylith folder structure.\n\n\n#### Create\n``` shell\npoetry poly create workspace --name my_namespace --theme <tdd or loose>\n```\n\n*New:* `theme` is a new Python Polylith feature and defines what kind of component structure - or theme - to use.\n\n`tdd` is the default and will set the structure according to the original Polylith Clojure implementation, such as:\n`components/<package>/src/<namespace>/<package>` with a corresponding `test` folder.\n\n`loose` is a new theme, for a more familiar structure for Python:\n`components/<namespace>/<package>` and will put a `test` folder at the root of the repository.\n\n\nAdd a component:\n\n``` shell\n# This command will create a component - i.e. a Python package in a namespaced folder.\npoetry poly create component --name my_component\n```\n\nAdd a base:\n\n``` shell\n# This command will create a base - i.e. a Python package in a namespaced folder.\npoetry poly create base --name my_example_aws_lambda\n```\n\nAdd a project:\n\n``` shell\n# This command will create a project - i.e. a pyproject.toml in a project folder. No code in this folder.\npoetry poly create project --name my_example_aws_lambada_project\n```\n\n#### Info\nShow info about the workspace:\n\n``` shell\npoetry poly info\n```\n\n#### Diff\nShows what has changed since the most recent stable point in time:\n\n``` shell\npoetry poly diff\n```\n\nThe `diff` command will compare the current state of the repository, compared to a `git tag`.\nThe tool will look for the latest tag according to a certain pattern, such as `stable-*`.\nThe pattern can be configured in `workspace.toml`.\n\nThe `diff` command is useful in a CI environment, to determine if a project should be deployed or not.\nThe command has a `--short` flag to only print a comma separated list of changed projects to the standard output.\n\n\nUseful for CI:\n``` shell\npoetry poly diff --short\n```\n\n#### Check\nValidates the Polylith workspace:\n\n``` shell\npoetry poly check\n```\n\n**NOTE**: this feature is built on top of the `poetry check-project` command from the [Multiproject](https://github.com/DavidVujic/poetry-multiproject-plugin) plugin.\nMake sure that you have the latest version of poetry-multiproject-plugin installed to be able to use the `poly check` command.\n\n#### Testing\nThe `create` commands will also create corresponding unit tests. It is possible to disable thi behaviour\nby setting `enabled = false` in the `workspace.toml` file.\n\n\n#### Workspace configuration\nAn example of a workspace configuration:\n\n``` toml\n[tool.polylith]\nnamespace = "my_namespace"\ngit_tag_pattern = "stable-*"\n\n[tool.polylith.structure]\ntheme = "loose"\n\n[tool.polylith.test]\nenabled = true\n```\n',
    'author': 'David Vujic',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/davidvujic/python-polylith',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
